"use strict";
/**
 * link-dependency-executables
 * Link a package's executables into node_modules
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.linkDependencyExecutables = void 0;
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const find_package_1 = require("@typescript-tools/find-package");
const io_ts_1 = require("@typescript-tools/io-ts");
const monorepo_root_1 = require("@typescript-tools/monorepo-root");
const debug_1 = __importDefault(require("debug"));
const Apply = __importStar(require("fp-ts/Apply"));
const E = __importStar(require("fp-ts/Either"));
const R = __importStar(require("fp-ts/Record"));
const TE = __importStar(require("fp-ts/TaskEither"));
const function_1 = require("fp-ts/function");
const PathReporter = __importStar(require("io-ts/lib/PathReporter"));
const debug = {
    cmd: (0, debug_1.default)('link'),
};
const findPackage = (packagePathOrName) => 
// FIXME: this should be exposed as a parameter
(0, find_package_1.findPackage)(process.cwd(), packagePathOrName);
const monorepoRoot = (0, function_1.flow)(monorepo_root_1.monorepoRoot, TE.fromEither);
const packageJson = (packageDirectory) => path.resolve(packageDirectory, 'package.json');
const topLevelDependency = (monorepoRoot) => (dependency) => path.resolve(monorepoRoot, 'node_modules', dependency);
const readFile = (filename) => TE.tryCatch(() => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => {
        fs.readFile(filename, 'utf8', (error, data) => {
            if (error !== null && error !== undefined) {
                reject(error);
            }
            else {
                resolve(data);
            }
        });
    });
}), (0, function_1.flow)(E.toError, (error) => ({ type: 'unable to read file', filename, error })));
const decodePackageJsonBin = (dependency) => (0, function_1.flow)((0, io_ts_1.StringifiedJSON)(io_ts_1.PackageJsonBin).decode.bind(null), E.mapLeft((0, function_1.flow)((errors) => PathReporter.failure(errors).join('\n'), (error) => ({
    type: 'unable to decode dependency package.json',
    dependency,
    error,
}))), TE.fromEither);
const externalPackageManifest = (monorepoRoot) => (dependencyName) => (0, function_1.pipe)(packageJson(topLevelDependency(monorepoRoot)(dependencyName)), readFile, TE.chainW(decodePackageJsonBin(dependencyName)));
const mkdir = (target) => (0, function_1.pipe)(TE.tryCatch(() => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => {
        fs.mkdir(target, { recursive: true }, (error) => {
            if (error !== null && error !== undefined) {
                reject(error);
            }
            else {
                resolve((0, function_1.constVoid)());
            }
        });
    });
}), (0, function_1.flow)(E.toError, (error) => ({ type: 'unable to create directory', path: target, error }))));
/**
 * Note: defaults to no action when `target` already exists.
 * Wonder if I should instead remove the existing file before creating `target`.
 */
const symlink = (target, link) => (0, function_1.pipe)(mkdir(path.dirname(link)), TE.chainW(() => (0, function_1.pipe)(TE.tryCatch(() => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => {
        fs.symlink(target, link, (error) => {
            if (error !== null && error !== undefined) {
                reject(error);
            }
            else {
                resolve((0, function_1.constVoid)());
            }
        });
    });
}), E.toError), 
// recover from symlink-already-exists error
TE.orElse((error) => error.message.startsWith('EEXIST:') ? TE.right((0, function_1.constVoid)()) : TE.left(error)), TE.mapLeft((error) => ({ type: 'unable to create symlink', target, path: link, error })))));
const symlinkExecutable = (monorepoRoot, internalPackageDirectory, dependencyName) => (executableName, targetPath) => {
    const symlinkLocation = path.resolve(internalPackageDirectory, 'node_modules', '.bin', executableName);
    const target = path.relative(path.dirname(symlinkLocation), path.resolve(topLevelDependency(monorepoRoot)(dependencyName), targetPath));
    return (0, function_1.pipe)(() => debug.cmd(`Creating symlink from ${symlinkLocation} -> ${target}`), TE.fromIO, TE.chain(() => symlink(target, symlinkLocation)));
};
const linkDependencyExecutables = (internalPackagePathOrName) => (dependencyName) => (0, function_1.pipe)(TE.bindTo('monorepoRoot')(monorepoRoot()), TE.bindW('internalPackage', () => findPackage(internalPackagePathOrName)), TE.bindW('dependencyManifest', ({ monorepoRoot }) => externalPackageManifest(monorepoRoot)(dependencyName)), TE.chainW(({ monorepoRoot, internalPackage, dependencyManifest }) => {
    var _a;
    return (0, function_1.pipe)(((_a = dependencyManifest.bin) !== null && _a !== void 0 ? _a : {}), R.mapWithIndex(symlinkExecutable(monorepoRoot, internalPackage.location, dependencyName)), (value) => value, Apply.sequenceS(TE.taskEither));
}), TE.map(function_1.constVoid));
exports.linkDependencyExecutables = linkDependencyExecutables;
//# sourceMappingURL=index.js.map